@@ -1,19 +1,8 @@
-﻿/********************************************************************
-Copyright 2019 Roman Gilg <subdiff@gmail.com>
+/*
+    SPDX-FileCopyrightText: 2019 Roman Gilg <subdiff@gmail.com>
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 #include "control.h"
 #include "globals.h"
 
@@ -676,6 +665,75 @@ void ControlConfig::setVrrPolicy(const QString &outputId, const QString &outputN
     setOutputVrr();
 }
 
+KScreen::Output::RgbRange ControlConfig::getRgbRange(const KScreen::OutputPtr &output) const
+{
+    return getRgbRange(output->hashMd5(), output->name());
+}
+
+KScreen::Output::RgbRange ControlConfig::getRgbRange(const QString &outputId, const QString &outputName) const
+{
+    const auto retention = getOutputRetention(outputId, outputName);
+    if (retention == OutputRetention::Individual) {
+        const QVariantList outputsInfo = getOutputs();
+        for (const auto &variantInfo : outputsInfo) {
+            const QVariantMap info = variantInfo.toMap();
+            if (!infoIsOutput(info, outputId, outputName)) {
+                continue;
+            }
+            const auto val = info[QStringLiteral("rgbrange")];
+            if (val.canConvert<uint>()) {
+                return static_cast<KScreen::Output::RgbRange>(val.toUInt());
+            } else {
+                return KScreen::Output::RgbRange::Automatic;
+            }
+        }
+    }
+    // Retention is global or info for output not in config control file.
+    if (auto *outputControl = getOutputControl(outputId, outputName)) {
+        return outputControl->rgbRange();
+    }
+
+    // Info for output not found.
+    return KScreen::Output::RgbRange::Automatic;
+}
+
+void ControlConfig::setRgbRange(const KScreen::OutputPtr &output, const KScreen::Output::RgbRange value)
+{
+    setRgbRange(output->hashMd5(), output->name(), value);
+}
+
+void ControlConfig::setRgbRange(const QString &outputId, const QString &outputName, const KScreen::Output::RgbRange enumValue)
+{
+    QList<QVariant>::iterator it;
+    QVariantList outputsInfo = getOutputs();
+    uint value = static_cast<uint>(enumValue);
+
+    auto setOutputRgbRange = [&outputId, &outputName, enumValue, this]() {
+        if (auto *control = getOutputControl(outputId, outputName)) {
+            control->setRgbRange(enumValue);
+        }
+    };
+
+    for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
+        QVariantMap outputInfo = (*it).toMap();
+        if (!infoIsOutput(outputInfo, outputId, outputName)) {
+            continue;
+        }
+        outputInfo[QStringLiteral("rgbrange")] = value;
+        *it = outputInfo;
+        setOutputs(outputsInfo);
+        setOutputRgbRange();
+        return;
+    }
+    // no entry yet, create one
+    auto outputInfo = createOutputInfo(outputId, outputName);
+    outputInfo[QStringLiteral("rgbrange")] = value;
+
+    outputsInfo << outputInfo;
+    setOutputs(outputsInfo);
+    setOutputRgbRange();
+}
+
 QVariantList ControlConfig::getOutputs() const
 {
     return constInfo()[QStringLiteral("outputs")].toList();
@@ -806,3 +864,21 @@ void ControlOutput::setVrrPolicy(KScreen::Output::VrrPolicy value)
     }
     infoMap[QStringLiteral("vrrpolicy")] = static_cast<uint>(value);
 }
+
+KScreen::Output::RgbRange ControlOutput::rgbRange() const
+{
+    const auto val = constInfo()[QStringLiteral("rgbrange")];
+    if (val.canConvert<uint>()) {
+        return static_cast<KScreen::Output::RgbRange>(val.toUInt());
+    }
+    return KScreen::Output::RgbRange::Automatic;
+}
+
+void ControlOutput::setRgbRange(KScreen::Output::RgbRange value)
+{
+    auto &infoMap = info();
+    if (infoMap.isEmpty()) {
+        infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
+    }
+    infoMap[QStringLiteral("rgbrange")] = static_cast<uint>(value);
+}
