@@ -1,19 +1,8 @@
-/********************************************************************
-Copyright © 2019 Roman Gilg <subdiff@gmail.com>
+/*
+    SPDX-FileCopyrightText: 2019 Roman Gilg <subdiff@gmail.com>
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 #include "output_model.h"
 
 #include "../common/utils.h"
@@ -92,6 +81,8 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
         return output->overscan();
     case VrrPolicyRole:
         return static_cast<uint32_t>(output->vrrPolicy());
+    case RgbRangeRole:
+        return static_cast<uint32_t>(output->rgbRange());
     }
     return QVariant();
 }
@@ -208,6 +199,18 @@ bool OutputModel::setData(const QModelIndex &index, const QVariant &value, int r
             return true;
         }
         break;
+    case RgbRangeRole:
+        if (value.canConvert<uint32_t>()) {
+            Output &output = m_outputs[index.row()];
+            const auto range = static_cast<KScreen::Output::RgbRange>(value.toUInt());
+            if (output.ptr->rgbRange() == range) {
+                return false;
+            }
+            output.ptr->setRgbRange(range);
+            m_config->setRgbRange(output.ptr, range);
+            Q_EMIT dataChanged(index, index, {role});
+            return true;
+        }
     }
     return false;
 }
@@ -235,6 +238,7 @@ QHash<int, QByteArray> OutputModel::roleNames() const
     roles[CapabilitiesRole] = "capabilities";
     roles[OverscanRole] = "overscan";
     roles[VrrPolicyRole] = "vrrPolicy";
+    roles[RgbRangeRole] = "rgbRange";
     return roles;
 }
 
@@ -268,6 +272,11 @@ void OutputModel::add(const KScreen::OutputPtr &output)
     });
     Q_EMIT endInsertRows();
 
+    connect(output.data(), &KScreen::Output::modesChanged, this, [this, output]() {
+        rolesChanged(output->id(), {ResolutionsRole, ResolutionIndexRole, SizeRole});
+        Q_EMIT sizeChanged();
+    });
+
     // Update replications.
     for (int j = 0; j < m_outputs.size(); j++) {
         if (i == j) {
@@ -588,6 +597,7 @@ QVector<float> OutputModel::refreshRates(const KScreen::OutputPtr &output) const
         }
         hits << rate;
     }
+    std::stable_sort(hits.begin(), hits.end(), std::greater<>());
     return hits;
 }
 
@@ -704,15 +714,27 @@ QVariantList OutputModel::replicasModel(const KScreen::OutputPtr &output) const
 }
 
 void OutputModel::roleChanged(int outputId, OutputRoles role)
+{
+    rolesChanged(outputId, {role});
+}
+
+void OutputModel::rolesChanged(int outputId, const QVector<int> &roles)
+{
+    const auto index = indexForOutputId(outputId);
+    if (index.isValid()) {
+        Q_EMIT dataChanged(index, index, roles);
+    }
+}
+
+QModelIndex OutputModel::indexForOutputId(int outputId) const
 {
     for (int i = 0; i < m_outputs.size(); i++) {
-        Output &output = m_outputs[i];
+        const Output &output = m_outputs[i];
         if (output.ptr->id() == outputId) {
-            QModelIndex index = createIndex(i, 0);
-            Q_EMIT dataChanged(index, index, {role});
-            return;
+            return createIndex(i, 0);
         }
     }
+    return QModelIndex();
 }
 
 bool OutputModel::positionable(const Output &output) const
